IterVariant

Enum IterVariant 

pub enum IterVariant<T: OutputIterator> {
    Before(T),
    After(T::AfterOutput),
}
Represents the internal state of an iterator over the OutputIterator. Used in various iterators of this crate.

Variants



Before(T)

The iterator is in its initial state, before producing any output.



After(T::AfterOutput)

The iterator has transitioned to the AfterOutput state.